#include "stdafx.h"
#include "common.h"

//
//		Y. Okamoto, 2009.10
//

namespace cmmn {

	int NCat;
	double * RatS, * RatN;
	double * mu, * sgm, ** ki;
	double med_mu, med_sgm, * med_ki, ** d_ki;
	double mean_mu, mean_sgm;

	using namespace System;

	double sqr( double x ){
			return x * x;
	}


/**************************************************************************

      Kerridge and Cook(1976, Biometrika, 63, 401-403)'s Algorithm
                          referred to
       in  A. L. Brophy and  D. L. Wood (1989, Behav. Res. Meth. Istr.
                                         & Comp., 21, 447-454)

**************************************************************************/

	double normal_0_to_z( double z ) {
		double p;
		z = Math::Abs(z);
		if (z > 6.8) {
			double zz = z * z;
			double d = zz + 3 - 1/(0.22 * zz + 0.704);
			double m = 1 - 1/d;
			p = 0.5 - Math::Exp(-0.5 * zz) * m / (z * Math::Sqrt(2 * Math::PI));
		} else {
			p = 0.0;
			if (z > 0.0) {
				double zz4 = 0.25 * z * z;
				double n2 = 2.0;                // initial value of 2n          ; n = 1
				double theta2 = 1.0;			// initial value of theta(2n-2)
				double theta1 = zz4;			// initial value of theta(2n-1)
				p = 1.0;						// sum for n = 0
				double p_prev = 0.0;			// initial value of sub-sum
				while (true) {
					theta2 = zz4 * (theta1 - theta2) / n2;
					n2 += 1.0;
					p += theta2/n2;
					if (p == p_prev) break;
					theta1 = zz4 * (theta2 - theta1) / n2;
					n2 += 1.0;
					p_prev = p;
				}
				p = z * Math::Exp(-0.5 * zz4) * p / Math::Sqrt(2.0 * Math::PI);
			};
		}
		return p;
	}


	double Cum_NormalKC( double z ) {
		if ( z >= 0.0 ) {
			if (z > 0.0) {
				double v = 0.5 + normal_0_to_z( z );
				if (v > 1.0) {
					return 1.0;
				} else {
					return v;
				};
			} else {
				return 0.5;
			};
		} else {
			double v = 0.5 - normal_0_to_z( Math::Abs(z) );
			if (v < 0.0){
				return 0.0;
			} else {
				return v;
			};
		}
	}


	double ProbS( int j, double mu, double sigma, double * ki ){
		if (j == 0){
				return Cum_NormalKC( (ki[0] - mu) / sigma );
		}else if(j == NCat-1){
				return 1.0 - Cum_NormalKC( (ki[NCat-2] - mu) / sigma );
		}else{
				return Cum_NormalKC( (ki[j] - mu) / sigma ) - Cum_NormalKC( (ki[j-1] - mu) / sigma );
		}
	}

	double ProbN( int j, double * ki ){
		if (j == 0){
				return Cum_NormalKC( ki[0] );
		}else if (j == NCat-1){
				return 1.0 - Cum_NormalKC( ki[NCat-2] );
		}else{
				return Cum_NormalKC( ki[j] ) - Cum_NormalKC( ki[j-1] );
		}
	}


	double calc_a( double mu1, double sgm1, double * ki0, double mu0, double sgm0 ){
			 double v = 0.0;
			 for (int j = 0; j < NCat; j++)
				 v += RatS[j] * (Math::Log(ProbS( j, mu1, sgm1, ki0 )) - Math::Log(ProbS( j, mu0, sgm0, ki0 )));
			 return Math::Exp(v);
	}

	double calc_a1( double mu, double sgm, double * kitmp, double * ki0 ){
			double v = 0.0;
			for (int j = 0; j < NCat; j++)
				v += RatS[j] * (Math::Log(ProbS( j, mu, sgm, kitmp )) - Math::Log(ProbS( j, mu, sgm, ki0 )))
						 + RatN[j] * (Math::Log(ProbN( j, kitmp )) - Math::Log(ProbN( j, ki0 )));
			return Math::Exp(v);
	}


	//
	//								Quick Sort
	//
	//      S. Kawai (1988).  Methods of programming (in Japanese). Tokyo: Iwanami-shoten.
	//		S. Ishihata (1989). Algorithms and Data Structure (in Japanese). Tokyo:Iwanami-shoten
	//
	void qSort( double * d, int Lb, int Ub ){
		if (Lb < Ub){
			int p = (Lb + Ub) / 2;
			double c = d[p];
			int i = Lb;
			int j = Ub;
			do {
				while (d[i] < c) i++;
				while (c < d[j]) j--;
				if (i <= j){
					double v = d[i];
					d[i] = d[j];
					d[j] = v;
					i++;
					j--;
				}
			} while (i <= j);
			qSort(d, Lb, j);
			qSort(d, i, Ub);
		}
		return;
	}


	double calc_QR( double * d, int N ){
		qSort( d, 0, N-1 );
		return 0.5 * (d[3*N/4] - d[N/4]);
	}

	double kernelF( double x, double sd ){
		return Math::Exp(-0.5 * sqr(x/sd));
	}


	double est_distri( double x, double * d, int N, double sd ){
		double v = 0.0;
		for (int i = 0; i < N; i++)
			v += kernelF( x - d[i], sd );
		return v / (double(N) * Math::Sqrt(2.0 * Math::PI * sd));
	}

	void calc_CI( double * d, int NSimu, double & CIL, double & Med, double & CIR ){
		Med = d[NSimu/2];
		double CIW = d[NSimu-1] - d[0];
		int posL, posR;
		for (int i = 0; i < int(NSimu*0.05); i++){
			if ((d[int(0.95*NSimu) + i] - d[i]) < CIW){
				CIW = d[int(0.95*NSimu) + i] - d[i];
				posR = int(0.95*NSimu) + i;
				posL = i;
			}
		}
		CIL = d[posL];
		CIR = d[posR];
		return;
	}

}
